uniform vec3 ambient;
uniform vec3 diffuse;
uniform vec3 specular;
uniform float shininess;
uniform float opacity;

uniform bool enableDiffuse;
uniform bool enableSpecular;
uniform bool enableAO;
uniform bool enableReflection;

uniform sampler2D tDiffuse;
uniform sampler2D tNormal;
uniform sampler2D tSpecular;
uniform sampler2D tAO;

uniform samplerCube tCube;

uniform vec2 uNormalScale;

uniform float refractionRatio;
uniform float reflectivity;

varying vec3 vTangent;
varying vec3 vBinormal;
varying vec3 vNormal;
varying vec2 vUv;

varying vec3 vWorldPosition;
varying vec3 vViewPosition;

#ifdef USE_FOG
	uniform vec3 fogColor;
	#ifdef FOG_EXP2
		uniform float fogDensity;
	#else
		uniform float fogNear;
		uniform float fogFar;
	#endif
#endif

#ifdef USE_LOGDEPTHBUF
	uniform float logDepthBufFC;
	#ifdef USE_LOGDEPTHBUF_EXT
		#extension GL_EXT_frag_depth : enable
		varying float vFragDepth;
	#endif
#endif

void main() {

	#if defined(USE_LOGDEPTHBUF) && defined(USE_LOGDEPTHBUF_EXT)
		gl_FragDepthEXT = log2(vFragDepth) * logDepthBufFC * 0.5;
	#endif

	gl_FragColor = vec4( vec3( 1.0 ), opacity );
	vec3 specularTex = vec3( 1.0 );
	vec3 normalTex = texture2D( tNormal, vUv ).xyz * 2.0 - 1.0;
	normalTex.xy *= uNormalScale;
	normalTex = normalize( normalTex );

	if( enableDiffuse ) {
		gl_FragColor = gl_FragColor * texture2D( tDiffuse, vUv );
	}
	if( enableAO ) {
		gl_FragColor.xyz = gl_FragColor.xyz * texture2D( tAO, vUv ).xyz;
	}
	if( enableSpecular ){
		specularTex = texture2D( tSpecular, vUv ).xyz;
	}

	mat3 tsb = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );
	vec3 finalNormal = tsb * normalTex;

	#ifdef FLIP_SIDED
		finalNormal = -finalNormal;
	#endif

	vec3 normal = normalize( finalNormal );
	vec3 viewPosition = normalize( vViewPosition );

	gl_FragColor.xyz = gl_FragColor.xyz ;

	if ( enableReflection ) {
		vec3 cameraToVertex = normalize( vWorldPosition - cameraPosition );
		#ifdef ENVMAP_MODE_REFLECTION
			vec3 vReflect = reflect( cameraToVertex, normal );
		#else
			// vec3 vReflect = refract( cameraToVertex, normal, 1. );
			vec3 vReflect = refract( cameraToVertex, normal, refractionRatio );
		#endif
		// vec4 cubeColor = textureCube( tCube, vec3( -vReflect.x, vReflect.yz ) );
		vec4 cubeColor = textureCube( tCube, -vReflect );

		gl_FragColor.xyz = mix( gl_FragColor.xyz, cubeColor.xyz, specularTex.r * reflectivity );
	}

	//============================
	// THREE.ShaderChunk[ "linear_to_gamma_fragment" ],
	//============================
	#ifdef GAMMA_OUTPUT
		gl_FragColor.xyz = sqrt( gl_FragColor.xyz );
	#endif
	//============================
	//============================
	//============================

	//============================
	// THREE.ShaderChunk[ "fog_fragment" ],
	//============================
	#ifdef USE_FOG
		#ifdef USE_LOGDEPTHBUF_EXT
			float depth = gl_FragDepthEXT / gl_FragCoord.w;
		#else
			float depth = gl_FragCoord.z / gl_FragCoord.w;
		#endif
		#ifdef FOG_EXP2
			const float LOG2 = 1.442695;
			float fogFactor = exp2( - fogDensity * fogDensity * depth * depth * LOG2 );
			fogFactor = 1.0 - clamp( fogFactor, 0.0, 1.0 );
		#else
			float fogFactor = smoothstep( fogNear, fogFar, depth );
		#endif
		gl_FragColor = mix( gl_FragColor, vec4( fogColor, gl_FragColor.w ), fogFactor );
	#endif
	//============================
	//============================
	//============================

	// gl_FragColor =  texture2D( tDiffuse, gl_FragCoord.xy*0.002 );
}	
